@@ -54,6 +54,10 @@ class XRefDummyCodeGenerator : public OutputCodeIntf
     OutputType type() const override { return OutputType::Extension; }
     std::unique_ptr<OutputCodeIntf> clone() override { return std::make_unique<XRefDummyCodeGenerator>(m_fd); }
     void codify(const QCString &) override {}
+    void stripCodeComments(bool) override {}
+    void startSpecialComment() override {}
+    void endSpecialComment() override {}
+    void setStripIndentAmount(size_t) override {}
     void writeCodeLink(CodeSymbolType,const QCString &,const QCString &,const QCString &,const QCString &,const QCString &) override  {}
     void writeLineNumber(const QCString &,const QCString &,const QCString &,int,bool) override {}
     virtual void writeTooltip(const QCString &,const DocLinkInfo &,
@@ -134,8 +138,8 @@ static void findXRefSymbols(FileDef *fd)
                 fileToString(fd->absFilePath()),
                 lang,
                 FALSE,
-                QCString(),
-                fd);
+                CodeParserOptions()
+                .setFileDef(fd));
 }
 
 static void listSymbol(Definition *d)
