@@ -18,7 +18,7 @@
 
 #include <memory>
 #include <vector>
-#include <set>
+#include <unordered_set>
 
 #include "containers.h"
 #include "definition.h"
@@ -92,7 +92,7 @@ using TemplateInstanceList = std::vector<TemplateInstanceDef>;
 
 using TemplateNameMap = std::map<std::string,int>;
 
-using ClassDefSet = std::set<const ClassDef*>;
+using ClassDefSet = std::unordered_set<const ClassDef*>;
 
 
 /** A abstract class representing of a compound symbol.
@@ -103,6 +103,8 @@ using ClassDefSet = std::set<const ClassDef*>;
 class ClassDef : public Definition
 {
   public:
+    ABSTRACT_BASE_CLASS(ClassDef)
+
     /** The various compound types */
     enum CompoundType { Class,     //=Entry::CLASS_SEC,
                         Struct,    //=Entry::STRUCT_SEC,
@@ -115,35 +117,21 @@ class ClassDef : public Definition
                         Singleton, //=Entry::CLASS_SEC
                       };
 
+    virtual std::unique_ptr<ClassDef> deepCopy(const QCString &name) const = 0;
+    virtual void moveTo(Definition *) = 0;
+
     //-----------------------------------------------------------------------------------
     // --- getters
     //-----------------------------------------------------------------------------------
 
-    /** Used for RTTI, this is a class */
-    virtual DefType definitionType() const = 0;
-
-    /** Returns the unique base name (without extension) of the class's file on disk */
-    virtual QCString getOutputFileBase() const = 0;
     virtual QCString getInstanceOutputFileBase() const = 0;
 
-    /** Returns the base name for the source code file */
-    virtual QCString getSourceFileBase() const = 0;
-
-    /** If this class originated from a tagfile, this will return the tag file reference */
-    virtual QCString getReference() const = 0;
-
-    /** Returns TRUE if this class is imported via a tag file */
-    virtual bool isReference() const = 0;
-
     /** Returns TRUE if this is a local class definition, see EXTRACT_LOCAL_CLASSES */
     virtual bool isLocal() const = 0;
 
     /** returns the classes nested into this class */
     virtual ClassLinkedRefMap getClasses() const = 0;
 
-    /** returns TRUE if this class has documentation */
-    virtual bool hasDocumentation() const = 0;
-
     /** returns TRUE if this class has a non-empty detailed description */
     virtual bool hasDetailedDescription() const = 0;
 
@@ -153,9 +141,6 @@ class ClassDef : public Definition
     /** returns the file name to use for the inheritance graph */
     virtual QCString inheritanceGraphFileName() const = 0;
 
-    /** Returns the name as it is appears in the documentation */
-    virtual QCString displayName(bool includeScope=TRUE) const = 0;
-
     /** Returns the type of compound this is, i.e. class/struct/union/.. */
     virtual CompoundType compoundType() const = 0;
 
@@ -187,15 +172,6 @@ class ClassDef : public Definition
      */
     virtual Protection protection() const = 0;
 
-    /** returns TRUE iff a link is possible to this item within this project.
-     */
-    virtual bool isLinkableInProject() const = 0;
-
-    /** return TRUE iff a link to this class is possible (either within
-     *  this project, or as a cross-reference to another project).
-     */
-    virtual bool isLinkable() const = 0;
-
     /** the class is visible in a class diagram, or class hierarchy */
     virtual bool isVisibleInHierarchy() const = 0;
 
@@ -262,12 +238,6 @@ class ClassDef : public Definition
 
     virtual bool isTemplateArgument() const = 0;
 
-    /** Returns the definition of a nested compound if
-     *  available, or 0 otherwise.
-     *  @param name The name of the nested compound
-     */
-    virtual const Definition *findInnerCompound(const QCString &name) const = 0;
-
     /** Returns the template parameter lists that form the template
      *  declaration of this class.
      *
@@ -278,7 +248,7 @@ class ClassDef : public Definition
     virtual ArgumentLists getTemplateParameterLists() const = 0;
 
     virtual QCString qualifiedNameWithTemplateParameters(
-        const ArgumentLists *actualParams=0,uint32_t *actualParamIndex=0) const = 0;
+        const ArgumentLists *actualParams=nullptr,uint32_t *actualParamIndex=nullptr) const = 0;
 
     /** Returns TRUE if there is at least one pure virtual member in this
      *  class.
@@ -333,7 +303,6 @@ class ClassDef : public Definition
 
     virtual bool isUsedOnly() const = 0;
 
-    virtual QCString anchor() const = 0;
     virtual bool isEmbeddedInOuterScope() const = 0;
 
     virtual bool isSimple() const = 0;
@@ -361,22 +330,18 @@ class ClassDef : public Definition
     virtual QCString requiresClause() const = 0;
     virtual StringVector getQualifiers() const = 0;
 
+    virtual bool containsOverload(const MemberDef *md) const = 0;
+
+    virtual bool isImplicitTemplateInstance() const = 0;
+
     //-----------------------------------------------------------------------------------
     // --- count members ----
     //-----------------------------------------------------------------------------------
 
     virtual int countMembersIncludingGrouped(MemberListType lt,
                 const ClassDef *inheritedFrom,bool additional) const = 0;
-    virtual int countInheritanceNodes() const = 0;
     virtual int countMemberDeclarations(MemberListType lt,const ClassDef *inheritedFrom,
-                int lt2,bool invert,bool showAlways,ClassDefSet &visitedClasses) const = 0;
-
-    //-----------------------------------------------------------------------------------
-    // --- helpers ----
-    //-----------------------------------------------------------------------------------
-
-    virtual ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
-                                const QCString &templSpec,bool &freshInstance) const = 0;
+                MemberListType lt2,bool invert,bool showAlways,ClassDefSet &visitedClasses) const = 0;
 
     //-----------------------------------------------------------------------------------
     // --- write output ----
@@ -392,13 +357,14 @@ class ClassDef : public Definition
                  int indentLevel, const ClassDef *inheritedFrom,const QCString &inheritId) const = 0;
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *md) const = 0;
     virtual void writeSummaryLinks(OutputList &ol) const = 0;
+    virtual void writePageNavigation(OutputList &ol) const = 0;
     virtual void writeInlineDocumentation(OutputList &ol) const = 0;
     virtual void writeTagFile(TextStream &) const = 0;
     virtual void writeMemberDeclarations(OutputList &ol,ClassDefSet &visitedClasses,
                  MemberListType lt,const QCString &title,
                  const QCString &subTitle=QCString(),
-                 bool showInline=FALSE,const ClassDef *inheritedFrom=0,
-                 int lt2=-1,bool invert=FALSE,bool showAlways=FALSE) const = 0;
+                 bool showInline=FALSE,const ClassDef *inheritedFrom=nullptr,
+                 MemberListType lt2=MemberListType::Invalid(),bool invert=FALSE,bool showAlways=FALSE) const = 0;
     virtual void addGroupedInheritedMembers(OutputList &ol,MemberListType lt,
                  const ClassDef *inheritedFrom,const QCString &inheritId) const = 0;
 };
@@ -406,6 +372,8 @@ class ClassDef : public Definition
 class ClassDefMutable : public DefinitionMutable, public ClassDef
 {
   public:
+    ABSTRACT_BASE_CLASS(ClassDefMutable)
+
     //-----------------------------------------------------------------------------------
     // --- setters ----
     //-----------------------------------------------------------------------------------
@@ -431,12 +399,20 @@ class ClassDefMutable : public DefinitionMutable, public ClassDef
     virtual void setRequiresClause(const QCString &req) = 0;
     virtual void addQualifiers(const StringVector &qualifiers) = 0;
         // inheritance graph related members
-    virtual CLASS_GRAPH_t inheritanceGraphType() const = 0;
-    virtual void setTypeInheritanceGraph(CLASS_GRAPH_t e) = 0;
+    virtual CLASS_GRAPH_t hasInheritanceGraph() const = 0;
+    virtual void overrideInheritanceGraph(CLASS_GRAPH_t e) = 0;
+    virtual void setImplicitTemplateInstance(bool b) = 0;
 
     // collaboration graph related members
     virtual bool hasCollaborationGraph() const = 0;
-    virtual void enableCollaborationGraph(bool e) = 0;
+    virtual void overrideCollaborationGraph(bool e) = 0;
+
+    //-----------------------------------------------------------------------------------
+    // --- helpers ----
+    //-----------------------------------------------------------------------------------
+
+    virtual ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
+                                const QCString &templSpec,bool &freshInstance) = 0;
 
     //-----------------------------------------------------------------------------------
     // --- actions ----
@@ -444,10 +420,11 @@ class ClassDefMutable : public DefinitionMutable, public ClassDef
 
     virtual void insertBaseClass(ClassDef *,const QCString &name,Protection p,Specifier s,const QCString &t=QCString()) = 0;
     virtual void insertSubClass(ClassDef *,Protection p,Specifier s,const QCString &t=QCString()) = 0;
+    virtual void insertExplicitTemplateInstance(ClassDef *instance,const QCString &spec) = 0;
     virtual void insertMember(MemberDef *) = 0;
     virtual void insertUsedFile(const FileDef *) = 0;
+    virtual void addMemberToTemplateInstance(const MemberDef *md, const ArgumentList &templateArguments, const QCString &templSpec) = 0;
     virtual void addMembersToTemplateInstance(const ClassDef *cd,const ArgumentList &templateArguments,const QCString &templSpec) = 0;
-    virtual void addInnerCompound(Definition *d) = 0;
     virtual bool addExample(const QCString &anchor,const QCString &name, const QCString &file) = 0;
     virtual void addUsedClass(ClassDef *cd,const QCString &accessName,Protection prot) = 0;
     virtual void addUsedByClass(ClassDef *cd,const QCString &accessName,Protection prot) = 0;
@@ -456,6 +433,7 @@ class ClassDefMutable : public DefinitionMutable, public ClassDef
     virtual void findSectionsInDocumentation() = 0;
     virtual void addMembersToMemberGroup() = 0;
     virtual void addListReferences() = 0;
+    virtual void addRequirementReferences() = 0;
     virtual void addTypeConstraints() = 0;
     virtual void computeAnchors() = 0;
     virtual void mergeMembers() = 0;
@@ -466,6 +444,7 @@ class ClassDefMutable : public DefinitionMutable, public ClassDef
     virtual void setAnonymousEnumType() = 0;
     virtual void countMembers() = 0;
     virtual void sortAllMembersList() = 0;
+    virtual void setPrimaryConstructorParams(const ArgumentList &list) = 0;
 
 
 };
@@ -505,12 +484,7 @@ Protection classInheritedProtectionLevel(const ClassDef *cd,const ClassDef *bcd,
  */
 struct UsesClassDef
 {
-  UsesClassDef(ClassDef *cd) : classDef(cd)
-  {
-  }
- ~UsesClassDef()
-  {
-  }
+  UsesClassDef(ClassDef *cd) : classDef(cd) {}
   void addAccessor(const QCString &s)
   {
     if (accessors.find(s.str())==accessors.end())
@@ -542,12 +516,7 @@ class UsesClassList : public std::vector<UsesClassDef>
  */
 struct ConstraintClassDef
 {
-  ConstraintClassDef(ClassDef *cd) : classDef(cd)
-  {
-  }
- ~ConstraintClassDef()
-  {
-  }
+  ConstraintClassDef(ClassDef *cd) : classDef(cd) {}
   void addAccessor(const QCString &s)
   {
     if (accessors.find(s.str())==accessors.end())
